@@ -162,9 +162,10 @@ dr.M.ols <- function(object,...) {
 #####################################################################
 
 dr.M.sir <-function(object,nslices=NULL,slice.function=dr.slices,sel=NULL,...) {
-    sel <- if(is.null(sel))1:length(dr.y(object)) else sel
+    sel <- if(is.null(sel))1:dim(dr.z(object))[1] else sel
     z <- dr.z(object)[sel,]
-    y <- dr.y(object)[sel]
+    y <- dr.y(object)
+    y <- if (is.matrix(y)) y[sel,] else y[sel]
 # get slice information    
     h <- if (!is.null(nslices)) nslices else max(8, NCOL(z)+3)
     slices <- slice.function(y,h)
@@ -177,7 +178,7 @@ dr.M.sir <-function(object,nslices=NULL,slice.function=dr.slices,sel=NULL,...) {
     wmean <- function (x, wts) { sum(x * wts) / sum (wts) }
     for (j in 1:slices$nslices){
       sel <- slices$slice.indicator==j
-      zmeans[j,]<- apply(z[sel,],2,wmean,wts[sel])
+      zmeans[j,]<- apply(z[sel,,drop=FALSE],2,wmean,wts[sel])
       slice.weight[j]<-sum(wts[sel])}
 # get M matrix for sir
     M <- t(zmeans) %*% apply(zmeans,2,"*",slice.weight)/ sum(slice.weight)
@@ -186,7 +187,7 @@ dr.M.sir <-function(object,nslices=NULL,slice.function=dr.slices,sel=NULL,...) {
 
 dr.M.msir <-function(...) {dr.M.sir(...)}
 
-dr.test.sir<-function(object,numdir=4,...) {
+dr.test.sir<-function(object,numdir=object$numdir,...) {
 #compute the sir test statistic for the first numdir directions
     e<-sort(object$evalues)
     p<-length(object$evalues)
@@ -256,9 +257,10 @@ dr.coordinate.test.sir<-function(object,hypothesis,d=NULL,
 #####################################################################
 
 dr.M.save<-function(object,nslices=NULL,slice.function=dr.slices,sel=NULL,...) {
-    sel <- if(is.null(sel)) 1:length(dr.y(object)) else sel
+    sel <- if(is.null(sel)) 1:dim(dr.z(object))[1] else sel
     z <- dr.z(object)[sel,]
-    y <- dr.y(object)[sel]
+    y <- dr.y(object)
+    y <- if (is.matrix(y)) y[sel,] else y[sel]
     wts <- dr.wts(object)[sel]
     h <- if (!is.null(nslices)) nslices else max(8, ncol(z) + 3)
     slices <- slice.function(y,h)
@@ -286,7 +288,7 @@ dr.M.save<-function(object,nslices=NULL,slice.function=dr.slices,sel=NULL,...) {
 }
 
 # Written by Yongwu Shao, 4/27/2006
-dr.test.save<-function(object,numdir,...) {
+dr.test.save<-function(object,numdir=object$numdir,...) {
     p<-length(object$evalues)
     n<-object$cases
     h<-object$slice.info$nslices
@@ -390,7 +392,7 @@ dr.y.phdres <- function(object) {
    qr.resid(object$qr,sw*(y-mean(y)))}
 dr.y.phd <- function(object) {dr.y.phdres(object)}
 
-dr.test.phd<-function(object,numdir,...) {
+dr.test.phd<-function(object,numdir=object$numdir,...) {
 # Modified by Jorge de la Vega, February, 2001
 #compute the phd asymptotic test statitics under restrictions for the
 #first numdir directions, based on response = OLS residuals
@@ -520,8 +522,9 @@ dr.direction  <- function(object, which, x) {UseMethod("dr.direction")}
 
 # standardize so that the first coordinates are always positive.
 dr.direction.default <- 
-  function(object, which=1:object$numdir,x=dr.x(object)) {
+  function(object, which=NULL,x=dr.x(object)) {  
     ans <- (apply(x,2,function(x){x-mean(x)}) %*% object$evectors)
+    which <- if (is.null(which)) seq(dim(ans)[2]) else which 
     ans <- apply(ans,2,function(x) if(x[1] <= 0)  -x else x)[,which]
     if (length(which) > 1) dimnames(ans) <-
                        list ( attr(x,"dimnames")[[1]],paste("Dir",which,sep=""))
@@ -570,15 +573,15 @@ function(z,use="color",values=NULL,color.fn=rainbow,na.action="na.use") {
 "print.dr" <-
 function(x, digits = max(3, getOption("digits") - 3), width=50, 
     numdir=x$numdir, ...)
-{
+{   
     fout <- deparse(x$call,width.cutoff=width)
     for (f in fout) cat("\n",f)
     cat("\n")
     cat("Estimated Basis Vectors for Central Subspace:\n")
     evectors<-x$evectors
-    print.default(evectors[,1:numdir])
+    print.default(evectors[,1:min(numdir,dim(evectors)[2])])
     cat("Eigenvalues:\n")
-    print.default(x$evalues[1:numdir])
+    print.default(x$evalues[1:min(numdir,dim(evectors)[2])])
     cat("\n")
     invisible(x)
 }
@@ -602,8 +605,8 @@ function(x, digits = max(3, getOption("digits") - 3), width=50,
     angles <- cosangle(dr.direction(object),ols.fit)
     angles <- if (is.matrix(angles)) angles[,1:nd] else angles[1:nd]
     if (is.matrix(angles)) dimnames(angles)[[1]] <- z$y.name
-    angle.names <- if (!is.matrix(angles)) "R^2(OLS|dr)" else
-                        paste("R^2(",dimnames(angles)[[1]],"|dr)",sep="")
+    angle.names <- if (!is.matrix(angles)) "R^2(OLS|dr)" else 
+          paste("R^2(OLS for ",dimnames(angles)[[1]],"|dr)",sep="")
     ans$evalues <-rbind (z$evalues[1:nd],angles)
     dimnames(ans$evalues)<-
      list(c("Eigenvalues",angle.names),
@@ -912,16 +915,18 @@ dr.slices <- function(y,nslices) {
        a$nslices <- ns }
 #recode unique values to 1:nslices and fix up slice sizes
     v <- unique(a$slice.indicator)
-    L <- NULL
+    L <- slice.indicator <- NULL
     for (i in 1:length(v)) {
        sel <- a$slice.indicator==v[i]
-       a$slice.indicator[sel] <- i
+       slice.indicator[sel] <- i
        L <- c(L,length(a$slice.indicator[sel]))}
+    a$slice.indicator <- slice.indicator
     a$slice.sizes <- L }
   a}
 
 dr.slices.arc<-function(y,nslices)  # matches slices produced by Arc
 {
+  if(is.matrix(y))  stop("dr.slices.arc is used for univariate y only.  Use dr.slices")
   h <- nslices
   or <- order(y)
   n <- length(y)
@@ -1037,52 +1042,91 @@ function (formula, data = list(), subset, na.action=na.fail,
 ###################################################################
 ## drop1 methods 
 ###################################################################
-    
 drop1.dr <-
-function (object, scope=NULL, update=TRUE, test = "general",...) 
-{
-    keep <- if(is.null(scope)) NULL else
-         attr(terms(update(object$terms,scope)),"term.labels")
-    all <- attr(object$terms,"term.labels")
-    candidates <- setdiff(all,keep)
-    if(length(candidates) == 0) stop("Error---nothing to drop")
+function (object, scope = NULL, update = TRUE, test = "general", 
+    trace=1, ...) 
+{               
+    keep <- if (is.null(scope)) 
+        NULL
+    else attr(terms(update(object$terms, scope)), "term.labels")
+    all <- attr(object$terms, "term.labels")
+    candidates <- setdiff(all, keep)
+    if (length(candidates) == 0) 
+        stop("Error---nothing to drop")
     ans <- NULL
-    for (label in candidates){
-     ans <- rbind(ans,
-      dr.coordinate.test(object,as.formula(paste("~.-",label,sep="")),...))
+    for (label in candidates) {
+        ans <- rbind(ans, dr.coordinate.test(object, as.formula(paste("~.-", 
+            label, sep = "")), ...))
     }
-    row.names(ans) <- paste("-",candidates)
+    row.names(ans) <- paste("-", candidates)
     ncols <- ncol(ans)
-    or <- order(-ans[,if(test=="general") ncols else (ncols-1)])
-    form <- formula(object) 
+    or <- order(-ans[, if (test == "general") 
+        ncols
+    else (ncols - 1)])
+    form <- formula(object)
     attributes(form) <- NULL
-    fout <- deparse(form,width.cutoff=50)
-    for (f in fout) cat("\n",f)
-    cat("\n")
-    print(ans[or,]) 
-    if (is.null(object$stop)) { object$stop <- 0 }
-    stopp <- if(ans[or[1],if(test=="general") ncols else (ncols-1)] <
-                  object$stop) TRUE else FALSE 
-    if (stopp == TRUE){ 
-     cat("\nStopping Criterion Met\n")
-     object$stop <- TRUE; object } else
-     if(update==TRUE) {
-      update(object,as.formula(paste("~.",row.names(ans)[or][1],sep="")))} 
-       else invisible(ans)
+    fout <- deparse(form, width.cutoff = 50)
+    if(trace > 0) {
+      for (f in fout) cat("\n", f)
+      cat("\n")
+      print(ans[or, ]) }
+    if (is.null(object$stop)) {
+        object$stop <- 0
     }
-  
+    stopp <- if (ans[or[1], if (test == "general") 
+        ncols
+    else (ncols - 1)] < object$stop) 
+        TRUE
+    else FALSE
+    if (stopp == TRUE) {
+        if(trace > 0) cat("\nStopping Criterion Met\n")
+        object$stop <- TRUE
+        object
+    }
+    else if (update == TRUE) {
+        update(object, as.formula(paste("~.", row.names(ans)[or][1], 
+            sep = "")))
+    }
+    else invisible(ans)
+}
+      
 dr.step <-
-function(object,scope=NULL,d=NULL,numdir=object$numdir,stop=0,...) {
-   if(is.null(object$stop)){ object$stop <- stop }
-   if(object$stop == TRUE) object else {
-    numdir <- max(2,numdir)
-    keep <- if(is.null(scope)) NULL else
-         attr(terms(update(object$terms,scope)),"term.labels")
-    all <- attr(object$terms,"term.labels")
-    if (length(keep) >= length(all) | length(all) <= numdir) object else {
-    if(dim(object$x)[2] <= numdir) object else {
-      obj1 <- drop1(object,scope=scope,d=d,...)
-      dr.step(obj1,scope=scope,d=d,numdir=numdir,stop=stop,...)}}}}
+function (object, scope = NULL, d = NULL, minsize = 2, stop = 0, 
+     trace=1,...) 
+{
+    if (is.null(object$stop)) {
+        object$stop <- stop
+    }
+    if (object$stop == TRUE) 
+        object
+    else {
+        minsize <- max(2, minsize)
+        keep <- if (is.null(scope)) 
+            NULL
+        else attr(terms(update(object$terms, scope)), "term.labels")
+        all <- attr(object$terms, "term.labels")
+        if (length(keep) >= length(all)) {
+            if(trace > 0) cat("\nNo more variables to remove\n")
+            object
+        }
+        else if (length(all) <= minsize) {
+            if (trace > 0) cat("\nMinimum size reached\n")
+            object$numdir <- minsize
+            object
+        }
+        else {
+            if (dim(object$x)[2] <= minsize) {
+                if (trace > 0) cat("\nMinimum size reached\n")
+                object}
+            else {   
+                obj1 <- drop1(object, scope = scope, d = d, trace=trace, ...)
+                dr.step(obj1, scope = scope, d = d, stop = stop,
+                       trace=trace, ...)
+            }
+        }
+    }
+}
+
  
 #####################################################################
 ##
@@ -1135,7 +1179,7 @@ dr.fit.ire <-function(object,numdir=4,nslices=NULL,slice.function=dr.slices,
     n <- object$cases
     weights <- object$weights
     p <- dim(z)[2]
-    numdir <- min(numdir,p-1)
+    numdir <- min(numdir,p-1)               
     h <- slices$nslices
     xi <- matrix(0,nrow=p,ncol=slices$nslices)
     for (j in 1:h){
@@ -1148,7 +1192,8 @@ dr.fit.ire <-function(object,numdir=4,nslices=NULL,slice.function=dr.slices,
     object$zeta <- xi %*% diag(f) %*% An
     rownames(object$zeta) <- paste("Q",1:dim(z)[2],sep="")
     ans <- NULL
-    if (tests == TRUE) {
+
+    if (tests == TRUE) {    
       object$indep.test <- dr.test(object,numdir=0,...)
       Gz <- Gzcomp(object,numdir)  # This is the same for all numdir > 0
       for (d in 1:numdir){
